@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,16 +23,13 @@
 
 #include <bson/bson-compat.h>
 #include <bson/bson-macros.h>
-#include <bson/bson-string.h>
+#include <common-string-private.h>
 
 
 BSON_BEGIN_DECLS
 
 bool
-_bson_iso8601_date_parse (const char *str,
-                          int32_t len,
-                          int64_t *out,
-                          bson_error_t *error);
+_bson_iso8601_date_parse (const char *str, int32_t len, int64_t *out, bson_error_t *error);
 
 /**
  * _bson_iso8601_date_format:
@@ -42,7 +39,7 @@ _bson_iso8601_date_parse (const char *str,
  * Appends a date formatted like "2012-12-24T12:15:30.500Z" to @str.
  */
 void
-_bson_iso8601_date_format (int64_t msecs_since_epoch, bson_string_t *str);
+_bson_iso8601_date_format (int64_t msecs_since_epoch, mcommon_string_t *str);
 
 BSON_END_DECLS
 
