@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,68 +30,53 @@
 BSON_BEGIN_DECLS
 
 
-#define BSON_ITER_HOLDS_DOUBLE(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_DOUBLE)
+#define BSON_ITER_HOLDS_DOUBLE(iter) (bson_iter_type ((iter)) == BSON_TYPE_DOUBLE)
 
 #define BSON_ITER_HOLDS_UTF8(iter) (bson_iter_type ((iter)) == BSON_TYPE_UTF8)
 
-#define BSON_ITER_HOLDS_DOCUMENT(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_DOCUMENT)
+#define BSON_ITER_HOLDS_DOCUMENT(iter) (bson_iter_type ((iter)) == BSON_TYPE_DOCUMENT)
 
 #define BSON_ITER_HOLDS_ARRAY(iter) (bson_iter_type ((iter)) == BSON_TYPE_ARRAY)
 
-#define BSON_ITER_HOLDS_BINARY(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_BINARY)
+#define BSON_ITER_HOLDS_BINARY(iter) (bson_iter_type ((iter)) == BSON_TYPE_BINARY)
 
-#define BSON_ITER_HOLDS_UNDEFINED(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_UNDEFINED)
+#define BSON_ITER_HOLDS_UNDEFINED(iter) (bson_iter_type ((iter)) == BSON_TYPE_UNDEFINED)
 
 #define BSON_ITER_HOLDS_OID(iter) (bson_iter_type ((iter)) == BSON_TYPE_OID)
 
 #define BSON_ITER_HOLDS_BOOL(iter) (bson_iter_type ((iter)) == BSON_TYPE_BOOL)
 
-#define BSON_ITER_HOLDS_DATE_TIME(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_DATE_TIME)
+#define BSON_ITER_HOLDS_DATE_TIME(iter) (bson_iter_type ((iter)) == BSON_TYPE_DATE_TIME)
 
 #define BSON_ITER_HOLDS_NULL(iter) (bson_iter_type ((iter)) == BSON_TYPE_NULL)
 
 #define BSON_ITER_HOLDS_REGEX(iter) (bson_iter_type ((iter)) == BSON_TYPE_REGEX)
 
-#define BSON_ITER_HOLDS_DBPOINTER(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_DBPOINTER)
+#define BSON_ITER_HOLDS_DBPOINTER(iter) (bson_iter_type ((iter)) == BSON_TYPE_DBPOINTER)
 
 #define BSON_ITER_HOLDS_CODE(iter) (bson_iter_type ((iter)) == BSON_TYPE_CODE)
 
-#define BSON_ITER_HOLDS_SYMBOL(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_SYMBOL)
+#define BSON_ITER_HOLDS_SYMBOL(iter) (bson_iter_type ((iter)) == BSON_TYPE_SYMBOL)
 
-#define BSON_ITER_HOLDS_CODEWSCOPE(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_CODEWSCOPE)
+#define BSON_ITER_HOLDS_CODEWSCOPE(iter) (bson_iter_type ((iter)) == BSON_TYPE_CODEWSCOPE)
 
 #define BSON_ITER_HOLDS_INT32(iter) (bson_iter_type ((iter)) == BSON_TYPE_INT32)
 
-#define BSON_ITER_HOLDS_TIMESTAMP(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_TIMESTAMP)
+#define BSON_ITER_HOLDS_TIMESTAMP(iter) (bson_iter_type ((iter)) == BSON_TYPE_TIMESTAMP)
 
 #define BSON_ITER_HOLDS_INT64(iter) (bson_iter_type ((iter)) == BSON_TYPE_INT64)
 
-#define BSON_ITER_HOLDS_DECIMAL128(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_DECIMAL128)
+#define BSON_ITER_HOLDS_DECIMAL128(iter) (bson_iter_type ((iter)) == BSON_TYPE_DECIMAL128)
 
-#define BSON_ITER_HOLDS_MAXKEY(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_MAXKEY)
+#define BSON_ITER_HOLDS_MAXKEY(iter) (bson_iter_type ((iter)) == BSON_TYPE_MAXKEY)
 
-#define BSON_ITER_HOLDS_MINKEY(iter) \
-   (bson_iter_type ((iter)) == BSON_TYPE_MINKEY)
+#define BSON_ITER_HOLDS_MINKEY(iter) (bson_iter_type ((iter)) == BSON_TYPE_MINKEY)
 
-#define BSON_ITER_HOLDS_INT(iter) \
-   (BSON_ITER_HOLDS_INT32 (iter) || BSON_ITER_HOLDS_INT64 (iter))
+#define BSON_ITER_HOLDS_INT(iter) (BSON_ITER_HOLDS_INT32 (iter) || BSON_ITER_HOLDS_INT64 (iter))
 
-#define BSON_ITER_HOLDS_NUMBER(iter) \
-   (BSON_ITER_HOLDS_INT (iter) || BSON_ITER_HOLDS_DOUBLE (iter))
+#define BSON_ITER_HOLDS_NUMBER(iter) (BSON_ITER_HOLDS_INT (iter) || BSON_ITER_HOLDS_DOUBLE (iter))
 
-#define BSON_ITER_IS_KEY(iter, key) \
-   (0 == strcmp ((key), bson_iter_key ((iter))))
+#define BSON_ITER_IS_KEY(iter, key) (0 == strcmp ((key), bson_iter_key ((iter))))
 
 
 BSON_EXPORT (const bson_value_t *)
@@ -110,7 +95,9 @@ static BSON_INLINE uint32_t
 bson_iter_utf8_len_unsafe (const bson_iter_t *iter)
 {
    uint32_t raw;
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&raw, iter->raw + iter->d1, sizeof (raw));
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 
    const uint32_t native = BSON_UINT32_FROM_LE (raw);
 
@@ -122,16 +109,11 @@ bson_iter_utf8_len_unsafe (const bson_iter_t *iter)
 
 
 BSON_EXPORT (void)
-bson_iter_array (const bson_iter_t *iter,
-                 uint32_t *array_len,
-                 const uint8_t **array);
+bson_iter_array (const bson_iter_t *iter, uint32_t *array_len, const uint8_t **array);
 
 
 BSON_EXPORT (void)
-bson_iter_binary (const bson_iter_t *iter,
-                  bson_subtype_t *subtype,
-                  uint32_t *binary_len,
-                  const uint8_t **binary);
+bson_iter_binary (const bson_iter_t *iter, bson_subtype_t *subtype, uint32_t *binary_len, const uint8_t **binary);
 
 
 BSON_EXPORT (const char *)
@@ -151,15 +133,14 @@ static BSON_INLINE const char *
 bson_iter_code_unsafe (const bson_iter_t *iter, uint32_t *length)
 {
    *length = bson_iter_utf8_len_unsafe (iter);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    return (const char *) (iter->raw + iter->d2);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 }
 
 
 BSON_EXPORT (const char *)
-bson_iter_codewscope (const bson_iter_t *iter,
-                      uint32_t *length,
-                      uint32_t *scope_len,
-                      const uint8_t **scope);
+bson_iter_codewscope (const bson_iter_t *iter, uint32_t *length, uint32_t *scope_len, const uint8_t **scope);
 
 
 BSON_EXPORT (void)
@@ -170,9 +151,7 @@ bson_iter_dbpointer (const bson_iter_t *iter,
 
 
 BSON_EXPORT (void)
-bson_iter_document (const bson_iter_t *iter,
-                    uint32_t *document_len,
-                    const uint8_t **document);
+bson_iter_document (const bson_iter_t *iter, uint32_t *document_len, const uint8_t **document);
 
 
 BSON_EXPORT (double)
@@ -194,7 +173,9 @@ bson_iter_double_unsafe (const bson_iter_t *iter)
 {
    double val;
 
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&val, iter->raw + iter->d1, sizeof (val));
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
    return BSON_DOUBLE_FROM_LE (val);
 }
 
@@ -203,9 +184,7 @@ BSON_EXPORT (bool)
 bson_iter_init (bson_iter_t *iter, const bson_t *bson);
 
 BSON_EXPORT (bool)
-bson_iter_init_from_data (bson_iter_t *iter,
-                          const uint8_t *data,
-                          size_t length);
+bson_iter_init_from_data (bson_iter_t *iter, const uint8_t *data, size_t length);
 
 
 BSON_EXPORT (bool)
@@ -213,23 +192,15 @@ bson_iter_init_find (bson_iter_t *iter, const bson_t *bson, const char *key);
 
 
 BSON_EXPORT (bool)
-bson_iter_init_find_w_len (bson_iter_t *iter,
-                           const bson_t *bson,
-                           const char *key,
-                           int keylen);
+bson_iter_init_find_w_len (bson_iter_t *iter, const bson_t *bson, const char *key, int keylen);
 
 
 BSON_EXPORT (bool)
-bson_iter_init_find_case (bson_iter_t *iter,
-                          const bson_t *bson,
-                          const char *key);
+bson_iter_init_find_case (bson_iter_t *iter, const bson_t *bson, const char *key);
 
 BSON_EXPORT (bool)
-bson_iter_init_from_data_at_offset (bson_iter_t *iter,
-                                    const uint8_t *data,
-                                    size_t length,
-                                    uint32_t offset,
-                                    uint32_t keylen);
+bson_iter_init_from_data_at_offset (
+   bson_iter_t *iter, const uint8_t *data, size_t length, uint32_t offset, uint32_t keylen);
 
 BSON_EXPORT (int32_t)
 bson_iter_int32 (const bson_iter_t *iter);
@@ -247,7 +218,9 @@ static BSON_INLINE int32_t
 bson_iter_int32_unsafe (const bson_iter_t *iter)
 {
    uint32_t raw;
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&raw, iter->raw + iter->d1, sizeof (raw));
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 
    const uint32_t native = BSON_UINT32_FROM_LE (raw);
 
@@ -277,7 +250,9 @@ static BSON_INLINE int64_t
 bson_iter_int64_unsafe (const bson_iter_t *iter)
 {
    uint64_t raw;
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&raw, iter->raw + iter->d1, sizeof (raw));
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 
    const uint64_t native = BSON_UINT64_FROM_LE (raw);
 
@@ -300,9 +275,7 @@ bson_iter_find_case (bson_iter_t *iter, const char *key);
 
 
 BSON_EXPORT (bool)
-bson_iter_find_descendant (bson_iter_t *iter,
-                           const char *dotkey,
-                           bson_iter_t *descendant);
+bson_iter_find_descendant (bson_iter_t *iter, const char *dotkey, bson_iter_t *descendant);
 
 
 BSON_EXPORT (bool)
@@ -324,7 +297,9 @@ bson_iter_oid (const bson_iter_t *iter);
 static BSON_INLINE const bson_oid_t *
 bson_iter_oid_unsafe (const bson_iter_t *iter)
 {
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    return (const bson_oid_t *) (iter->raw + iter->d1);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 }
 
 
@@ -346,8 +321,10 @@ bson_iter_decimal128_unsafe (const bson_iter_t *iter, bson_decimal128_t *dec)
    uint64_t low_le;
    uint64_t high_le;
 
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&low_le, iter->raw + iter->d1, sizeof (low_le));
    memcpy (&high_le, iter->raw + iter->d1 + 8, sizeof (high_le));
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 
    dec->low = BSON_UINT64_FROM_LE (low_le);
    dec->high = BSON_UINT64_FROM_LE (high_le);
@@ -372,7 +349,9 @@ bson_iter_key_len (const bson_iter_t *iter);
 static BSON_INLINE const char *
 bson_iter_key_unsafe (const bson_iter_t *iter)
 {
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    return (const char *) (iter->raw + iter->key);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 }
 
 
@@ -391,7 +370,9 @@ static BSON_INLINE const char *
 bson_iter_utf8_unsafe (const bson_iter_t *iter, size_t *length)
 {
    *length = bson_iter_utf8_len_unsafe (iter);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    return (const char *) (iter->raw + iter->d2);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 }
 
 
@@ -449,9 +430,7 @@ bson_iter_timeval_unsafe (const bson_iter_t *iter, struct timeval *tv)
 
 
 BSON_EXPORT (void)
-bson_iter_timestamp (const bson_iter_t *iter,
-                     uint32_t *timestamp,
-                     uint32_t *increment);
+bson_iter_timestamp (const bson_iter_t *iter, uint32_t *timestamp, uint32_t *increment);
 
 
 BSON_EXPORT (bool)
@@ -471,7 +450,9 @@ bson_iter_bool_unsafe (const bson_iter_t *iter)
 {
    char val;
 
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    memcpy (&val, iter->raw + iter->d1, 1);
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
    return !!val;
 }
 
@@ -503,7 +484,9 @@ bson_iter_type (const bson_iter_t *iter);
 static BSON_INLINE bson_type_t
 bson_iter_type_unsafe (const bson_iter_t *iter)
 {
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_BEGIN
    return (bson_type_t) (iter->raw + iter->type)[0];
+   BSON_DISABLE_UNSAFE_BUFFER_USAGE_WARNING_END
 }
 
 
@@ -524,8 +507,7 @@ bson_iter_overwrite_double (bson_iter_t *iter, double value);
 
 
 BSON_EXPORT (void)
-bson_iter_overwrite_decimal128 (bson_iter_t *iter,
-                                const bson_decimal128_t *value);
+bson_iter_overwrite_decimal128 (bson_iter_t *iter, const bson_decimal128_t *value);
 
 
 BSON_EXPORT (void)
@@ -537,9 +519,7 @@ bson_iter_overwrite_oid (bson_iter_t *iter, const bson_oid_t *value);
 
 
 BSON_EXPORT (void)
-bson_iter_overwrite_timestamp (bson_iter_t *iter,
-                               uint32_t timestamp,
-                               uint32_t increment);
+bson_iter_overwrite_timestamp (bson_iter_t *iter, uint32_t timestamp, uint32_t increment);
 
 
 BSON_EXPORT (void)
@@ -547,9 +527,7 @@ bson_iter_overwrite_date_time (bson_iter_t *iter, int64_t value);
 
 
 BSON_EXPORT (bool)
-bson_iter_visit_all (bson_iter_t *iter,
-                     const bson_visitor_t *visitor,
-                     void *data);
+bson_iter_visit_all (bson_iter_t *iter, const bson_visitor_t *visitor, void *data);
 
 BSON_EXPORT (uint32_t)
 bson_iter_offset (bson_iter_t *iter);
