@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,8 +18,8 @@
 #include "common-config.h"
 #include "common-macros-private.h"
 
-#ifndef COMMON_THREAD_PRIVATE_H
-#define COMMON_THREAD_PRIVATE_H
+#ifndef MONGO_C_DRIVER_COMMON_THREAD_PRIVATE_H
+#define MONGO_C_DRIVER_COMMON_THREAD_PRIVATE_H
 
 #define BSON_INSIDE
 #include "bson/bson-compat.h"
@@ -44,8 +44,7 @@ BSON_BEGIN_DECLS
    } while (0)
 #define bson_once_t pthread_once_t
 #define bson_thread_t pthread_t
-#define BSON_THREAD_FUN(_function_name, _arg_name) \
-   void *(_function_name) (void *(_arg_name))
+#define BSON_THREAD_FUN(_function_name, _arg_name) void *(_function_name) (void *(_arg_name))
 #define BSON_THREAD_FUN_TYPE(_function_name) void *(*(_function_name)) (void *)
 #define BSON_THREAD_RETURN return NULL
 
@@ -114,8 +113,7 @@ typedef struct {
 
 #else
 #include <process.h>
-#define BSON_ONCE_FUN(n) \
-   BOOL CALLBACK n (PINIT_ONCE _ignored_a, PVOID _ignored_b, PVOID *_ignored_c)
+#define BSON_ONCE_FUN(n) BOOL CALLBACK n (PINIT_ONCE _ignored_a, PVOID _ignored_b, PVOID *_ignored_c)
 #define BSON_ONCE_INIT INIT_ONCE_STATIC_INIT
 #define BSON_ONCE_RETURN return true
 #define bson_mutex_destroy DeleteCriticalSection
@@ -129,10 +127,8 @@ typedef struct {
    } while (0)
 #define bson_once_t INIT_ONCE
 #define bson_thread_t HANDLE
-#define BSON_THREAD_FUN(_function_name, _arg_name) \
-   unsigned (__stdcall _function_name) (void *(_arg_name))
-#define BSON_THREAD_FUN_TYPE(_function_name) \
-   unsigned (__stdcall * _function_name) (void *)
+#define BSON_THREAD_FUN(_function_name, _arg_name) unsigned (__stdcall _function_name) (void *(_arg_name))
+#define BSON_THREAD_FUN_TYPE(_function_name) unsigned (__stdcall * _function_name) (void *)
 #define BSON_THREAD_RETURN return 0
 #endif
 
@@ -145,9 +141,7 @@ mcommon_thread_join (bson_thread_t thread);
 // error. Callers may use `bson_strerror_r` to get an error message from the
 // returned error code.
 int
-mcommon_thread_create (bson_thread_t *thread,
-                       BSON_THREAD_FUN_TYPE (func),
-                       void *arg);
+mcommon_thread_create (bson_thread_t *thread, BSON_THREAD_FUN_TYPE (func), void *arg);
 
 #if defined(MONGOC_ENABLE_DEBUG_ASSERTIONS) && defined(BSON_OS_UNIX)
 #define mcommon_mutex_is_locked COMMON_NAME (mutex_is_locked)
@@ -212,4 +206,4 @@ bson_shared_mutex_unlock_shared (bson_shared_mutex_t *mtx)
 
 BSON_END_DECLS
 
-#endif /* COMMON_THREAD_PRIVATE_H */
+#endif /* MONGO_C_DRIVER_COMMON_THREAD_PRIVATE_H */
