@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,14 +14,12 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_ASYNC_PRIVATE_H
 #define MONGOC_ASYNC_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
-#include <bson.h>
+#include <bson/bson.h>
 #include "mongoc-stream.h"
 
 BSON_BEGIN_DECLS
@@ -45,20 +43,16 @@ typedef enum {
 typedef void (*mongoc_async_cmd_cb_t) (struct _mongoc_async_cmd *acmd,
                                        mongoc_async_cmd_result_t result,
                                        const bson_t *bson,
-                                       int64_t rtt_msec);
+                                       int64_t duration_usec);
 
-typedef mongoc_stream_t *(*mongoc_async_cmd_initiate_t) (
-   struct _mongoc_async_cmd *);
+typedef mongoc_stream_t *(*mongoc_async_cmd_initiate_t) (struct _mongoc_async_cmd *);
 
-typedef int (*mongoc_async_cmd_setup_t) (mongoc_stream_t *stream,
-                                         int *events,
-                                         void *ctx,
-                                         int32_t timeout_msec,
-                                         bson_error_t *error);
+typedef int (*mongoc_async_cmd_setup_t) (
+   mongoc_stream_t *stream, int *events, void *ctx, int32_t timeout_msec, bson_error_t *error);
 
 
 mongoc_async_t *
-mongoc_async_new ();
+mongoc_async_new (void);
 
 void
 mongoc_async_destroy (mongoc_async_t *async);
