@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,8 +41,6 @@
 
 BSON_BEGIN_DECLS
 
-/* version corresponding to server 3.6 release */
-#define WIRE_VERSION_3_6 6
 /* version corresponding to server 4.0 release */
 #define WIRE_VERSION_4_0 7
 /* first version to support hint for "update" command */
@@ -79,12 +77,14 @@ BSON_BEGIN_DECLS
 /* version corresponding to server 7.1 release */
 #define WIRE_VERSION_7_1 22
 #define WIRE_VERSION_MONGOS_EXHAUST 22
+/* version corresponding to server 8.0 release */
+#define WIRE_VERSION_8_0 25
 
 /* Range of wire protocol versions this driver supports. Bumping
  * WIRE_VERSION_MAX must be accompanied by an update to
  * `_mongoc_wire_version_to_server_version`. */
-#define WIRE_VERSION_MIN WIRE_VERSION_3_6 /* a.k.a. minWireVersion */
-#define WIRE_VERSION_MAX WIRE_VERSION_7_0 /* a.k.a. maxWireVersion */
+#define WIRE_VERSION_MIN WIRE_VERSION_4_0 /* a.k.a. minWireVersion */
+#define WIRE_VERSION_MAX WIRE_VERSION_8_0 /* a.k.a. maxWireVersion */
 
 struct _mongoc_collection_t;
 
@@ -134,8 +134,7 @@ typedef enum {
    MONGOC_CMD_RW = 3,
 } mongoc_command_mode_t;
 
-BSON_STATIC_ASSERT2 (mongoc_cmd_rw,
-                     MONGOC_CMD_RW == (MONGOC_CMD_READ | MONGOC_CMD_WRITE));
+BSON_STATIC_ASSERT2 (mongoc_cmd_rw, MONGOC_CMD_RW == (MONGOC_CMD_READ | MONGOC_CMD_WRITE));
 
 
 /* TODO (CDRIVER-4052): Move MONGOC_RR_DEFAULT_BUFFER_SIZE and
@@ -147,15 +146,14 @@ _mongoc_client_get_rr (const char *hostname,
                        mongoc_rr_type_t rr_type,
                        mongoc_rr_data_t *rr_data,
                        size_t initial_buffer_size,
+                       bool prefer_tcp,
                        bson_error_t *error);
 
 mongoc_client_t *
 _mongoc_client_new_from_topology (mongoc_topology_t *topology);
 
 bool
-_mongoc_client_set_apm_callbacks_private (mongoc_client_t *client,
-                                          mongoc_apm_callbacks_t *callbacks,
-                                          void *context);
+_mongoc_client_set_apm_callbacks_private (mongoc_client_t *client, mongoc_apm_callbacks_t *callbacks, void *context);
 
 mongoc_stream_t *
 mongoc_client_default_stream_initiator (const mongoc_uri_t *uri,
@@ -164,9 +162,7 @@ mongoc_client_default_stream_initiator (const mongoc_uri_t *uri,
                                         bson_error_t *error);
 
 mongoc_stream_t *
-_mongoc_client_create_stream (mongoc_client_t *client,
-                              const mongoc_host_list_t *host,
-                              bson_error_t *error);
+_mongoc_client_create_stream (mongoc_client_t *client, const mongoc_host_list_t *host, bson_error_t *error);
 
 bool
 _mongoc_client_recv (mongoc_client_t *client,
@@ -198,8 +194,7 @@ _mongoc_client_command_with_opts (mongoc_client_t *client,
                                   bson_error_t *error);
 
 mongoc_server_session_t *
-_mongoc_client_pop_server_session (mongoc_client_t *client,
-                                   bson_error_t *error);
+_mongoc_client_pop_server_session (mongoc_client_t *client, bson_error_t *error);
 
 bool
 _mongoc_client_lookup_session (const mongoc_client_t *client,
@@ -208,19 +203,15 @@ _mongoc_client_lookup_session (const mongoc_client_t *client,
                                bson_error_t *error);
 
 void
-_mongoc_client_unregister_session (mongoc_client_t *client,
-                                   mongoc_client_session_t *session);
+_mongoc_client_unregister_session (mongoc_client_t *client, mongoc_client_session_t *session);
 
 void
-_mongoc_client_push_server_session (mongoc_client_t *client,
-                                    mongoc_server_session_t *server_session);
+_mongoc_client_push_server_session (mongoc_client_t *client, mongoc_server_session_t *server_session);
 void
 _mongoc_client_end_sessions (mongoc_client_t *client);
 
 mongoc_stream_t *
-mongoc_client_connect_tcp (int32_t connecttimeoutms,
-                           const mongoc_host_list_t *host,
-                           bson_error_t *error);
+mongoc_client_connect_tcp (int32_t connecttimeoutms, const mongoc_host_list_t *host, bson_error_t *error);
 
 mongoc_stream_t *
 mongoc_client_connect (bool buffered,
@@ -228,6 +219,7 @@ mongoc_client_connect (bool buffered,
                        void *ssl_opts_void,
                        const mongoc_uri_t *uri,
                        const mongoc_host_list_t *host,
+                       void *openssl_ctx_void,
                        bson_error_t *error);
 
 
