@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,17 +14,20 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_CLIENT_SESSION_PRIVATE_H
 #define MONGOC_CLIENT_SESSION_PRIVATE_H
 
 #include <bson/bson.h>
-#include "mongoc/mongoc-client-session.h"
+#include "mongoc-client-session.h"
 
 /* error labels: see Transactions Spec */
 #define TRANSIENT_TXN_ERR "TransientTransactionError"
 #define UNKNOWN_COMMIT_RESULT "UnknownTransactionCommitResult"
+#define MAX_TIME_MS_EXPIRED "MaxTimeMSExpired"
+#define DEFAULT_MAX_COMMIT_TIME_MS 0
+#define SESSION_NEVER_USED (-1)
 
 #define MONGOC_DEFAULT_WTIMEOUT_FOR_COMMIT_RETRY 10000
 
@@ -32,37 +35,34 @@ struct _mongoc_transaction_opt_t {
    mongoc_read_concern_t *read_concern;
    mongoc_write_concern_t *write_concern;
    mongoc_read_prefs_t *read_prefs;
+   int64_t max_commit_time_ms;
 };
 
-typedef enum {
-   MONGOC_SESSION_NO_OPTS = 0,
-   MONGOC_SESSION_CAUSAL_CONSISTENCY = (1 << 0),
-} mongoc_session_flag_t;
-
 struct _mongoc_session_opt_t {
-   mongoc_session_flag_t flags;
+   mongoc_optional_t causal_consistency;
+   mongoc_optional_t snapshot;
    mongoc_transaction_opt_t default_txn_opts;
 };
 
 typedef struct _mongoc_server_session_t {
-   struct _mongoc_server_session_t *prev, *next;
    int64_t last_used_usec;
    bson_t lsid;        /* logical session id */
    int64_t txn_number; /* transaction number */
+   bool dirty;
 } mongoc_server_session_t;
 
 typedef enum {
-   MONGOC_TRANSACTION_NONE,
-   MONGOC_TRANSACTION_STARTING,
-   MONGOC_TRANSACTION_IN_PROGRESS,
-   MONGOC_TRANSACTION_ENDING,
-   MONGOC_TRANSACTION_COMMITTED,
-   MONGOC_TRANSACTION_COMMITTED_EMPTY,
-   MONGOC_TRANSACTION_ABORTED,
-} mongoc_transaction_state_t;
+   MONGOC_INTERNAL_TRANSACTION_NONE,
+   MONGOC_INTERNAL_TRANSACTION_STARTING,
+   MONGOC_INTERNAL_TRANSACTION_IN_PROGRESS,
+   MONGOC_INTERNAL_TRANSACTION_ENDING,
+   MONGOC_INTERNAL_TRANSACTION_COMMITTED,
+   MONGOC_INTERNAL_TRANSACTION_COMMITTED_EMPTY,
+   MONGOC_INTERNAL_TRANSACTION_ABORTED,
+} mongoc_internal_transaction_state_t;
 
 typedef struct _mongoc_transaction_t {
-   mongoc_transaction_state_t state;
+   mongoc_internal_transaction_state_t state;
    mongoc_transaction_opt_t opts;
 } mongoc_transaction_t;
 
@@ -76,12 +76,19 @@ struct _mongoc_client_session_t {
    uint32_t operation_timestamp;
    uint32_t operation_increment;
    uint32_t client_generation;
+   uint32_t server_id;
+   bson_t *recovery_token;
+   uint32_t snapshot_time_timestamp;
+   uint32_t snapshot_time_increment;
+   bool snapshot_time_set;
+
+   /* For testing only */
+   int64_t with_txn_timeout_ms;
+   const char *fail_commit_label;
 };
 
 bool
-_mongoc_parse_cluster_time (const bson_t *cluster_time,
-                            uint32_t *timestamp,
-                            uint32_t *increment);
+_mongoc_parse_cluster_time (const bson_t *cluster_time, uint32_t *timestamp, uint32_t *increment);
 
 bool
 _mongoc_cluster_time_greater (const bson_t *new, const bson_t *old);
@@ -89,17 +96,17 @@ _mongoc_cluster_time_greater (const bson_t *new, const bson_t *old);
 void
 _mongoc_client_session_handle_reply (mongoc_client_session_t *session,
                                      bool is_acknowledged,
+                                     const char *cmd_name,
                                      const bson_t *reply);
 
-mongoc_server_session_t *
-_mongoc_server_session_new (bson_error_t *error);
-
 bool
-_mongoc_server_session_timed_out (const mongoc_server_session_t *server_session,
-                                  int64_t session_timeout_minutes);
+_mongoc_server_session_init (mongoc_server_session_t *session, bson_error_t *error);
 
 void
-_mongoc_server_session_destroy (mongoc_server_session_t *server_session);
+_mongoc_server_session_destroy (mongoc_server_session_t *session);
+
+bool
+_mongoc_server_session_timed_out (const mongoc_server_session_t *server_session, int64_t session_timeout_minutes);
 
 mongoc_client_session_t *
 _mongoc_client_session_new (mongoc_client_t *client,
@@ -116,13 +123,14 @@ _mongoc_client_session_from_iter (mongoc_client_t *client,
 bool
 _mongoc_client_session_in_txn (const mongoc_client_session_t *session);
 
+bool
+_mongoc_client_session_in_txn_or_ending (const mongoc_client_session_t *session);
+
 bool
 _mongoc_client_session_txn_in_progress (const mongoc_client_session_t *session);
 
 bool
-_mongoc_client_session_append_txn (mongoc_client_session_t *session,
-                                   bson_t *cmd,
-                                   bson_error_t *error);
+_mongoc_client_session_append_txn (mongoc_client_session_t *session, bson_t *cmd, bson_error_t *error);
 
 void
 _mongoc_client_session_append_read_concern (const mongoc_client_session_t *cs,
@@ -130,4 +138,16 @@ _mongoc_client_session_append_read_concern (const mongoc_client_session_t *cs,
                                             bool is_read_command,
                                             bson_t *cmd);
 
+void
+_mongoc_client_session_unpin (mongoc_client_session_t *session);
+
+void
+_mongoc_client_session_pin (mongoc_client_session_t *session, uint32_t server_id);
+
+void
+_mongoc_client_session_set_snapshot_time (mongoc_client_session_t *session, uint32_t t, uint32_t i);
+
+void
+_mongoc_client_session_clear_snapshot_time (mongoc_client_session_t *session);
+
 #endif /* MONGOC_CLIENT_SESSION_PRIVATE_H */
