@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,26 +14,18 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_CLUSTER_SASL_PRIVATE_H
 #define MONGOC_CLUSTER_SASL_PRIVATE_H
 
-#if !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
 #include "mongoc-config.h"
 #include "mongoc-cluster-private.h"
-#include <bson.h>
+#include <bson/bson.h>
 
 bool
 _mongoc_cluster_auth_node_sasl (mongoc_cluster_t *cluster,
                                 mongoc_stream_t *stream,
-                                const char *hostname,
+                                mongoc_server_description_t *sd,
                                 bson_error_t *error);
-bool
-_mongoc_cluster_get_canonicalized_name (mongoc_cluster_t *cluster,    /* IN */
-                                        mongoc_stream_t *node_stream, /* IN */
-                                        char *name,                   /* OUT */
-                                        size_t namelen,               /* IN */
-                                        bson_error_t *error);         /* OUT */
 #endif /* MONGOC_CLUSTER_SASL_PRIVATE_H */
