@@ -1,5 +1,5 @@
 /*
- * Copyright 2017 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
  */
 
 /* for size_t */
-#include <bson.h>
+#include <bson/bson.h>
 #include "mongoc-config.h"
 
 #include "mongoc-cluster-private.h"
@@ -34,26 +34,19 @@
 #ifdef MONGOC_ENABLE_SASL_SSPI
 #include "mongoc-cluster-sspi-private.h"
 #endif
-#ifdef MONGOC_ENABLE_SASL_GSSAPI
-#include "mongoc-cluster-gssapi-private.h"
+
 #endif
 
 void
-_mongoc_cluster_build_sasl_start (bson_t *cmd,
-                                  const char *mechanism,
-                                  const char *buf,
-                                  uint32_t buflen)
+_mongoc_cluster_build_sasl_start (bson_t *cmd, const char *mechanism, const char *buf, uint32_t buflen)
 {
    BSON_APPEND_INT32 (cmd, "saslStart", 1);
-   BSON_APPEND_UTF8 (cmd, "mechanism", "GSSAPI");
+   BSON_APPEND_UTF8 (cmd, "mechanism", mechanism);
    bson_append_utf8 (cmd, "payload", 7, buf, buflen);
    BSON_APPEND_INT32 (cmd, "autoAuthorize", 1);
 }
 void
-_mongoc_cluster_build_sasl_continue (bson_t *cmd,
-                                     int conv_id,
-                                     const char *buf,
-                                     uint32_t buflen)
+_mongoc_cluster_build_sasl_continue (bson_t *cmd, int conv_id, const char *buf, uint32_t buflen)
 {
    BSON_APPEND_INT32 (cmd, "saslContinue", 1);
    BSON_APPEND_INT32 (cmd, "conversationId", conv_id);
@@ -64,14 +57,12 @@ _mongoc_cluster_get_conversation_id (const bson_t *reply)
 {
    bson_iter_t iter;
 
-   if (bson_iter_init_find (&iter, reply, "conversationId") &&
-       BSON_ITER_HOLDS_INT32 (&iter)) {
+   if (bson_iter_init_find (&iter, reply, "conversationId") && BSON_ITER_HOLDS_INT32 (&iter)) {
       return bson_iter_int32 (&iter);
    }
 
    return 0;
 }
-#endif
 
 /*
  *--------------------------------------------------------------------------
@@ -107,7 +98,5 @@ _mongoc_cluster_auth_node_sasl (mongoc_cluster_t *cluster,
    return _mongoc_cluster_auth_node_cyrus (cluster, stream, sd, error);
 #elif defined(MONGOC_ENABLE_SASL_SSPI)
    return _mongoc_cluster_auth_node_sspi (cluster, stream, sd, error);
-#elif defined(MONGOC_ENABLE_SASL_GSSAPI)
-   return _mongoc_cluster_auth_node_gssapi (cluster, stream, sd, error);
 #endif
 }
