@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 
 #include "mongoc-counters-private.h"
 #include "mongoc-log.h"
+#include <common-atomic-private.h>
 
 
 #pragma pack(1)
@@ -71,8 +72,7 @@ BSON_STATIC_ASSERT2 (counters_t, sizeof (mongoc_counters_t) == 64);
  * for whether or not initiating the shared memory segment succeeded. */
 static void *gCounterFallback = NULL;
 
-#define COUNTER(ident, Category, Name, Description) \
-   mongoc_counter_t __mongoc_counter_##ident;
+#define COUNTER(ident, Category, Name, Description) mongoc_counter_t __mongoc_counter_##ident;
 #include "mongoc-counters.defs"
 #undef COUNTER
 
@@ -107,8 +107,7 @@ mongoc_counters_calc_size (void)
 
    n_cpu = _mongoc_get_cpu_count ();
    n_groups = (LAST_COUNTER / SLOTS_PER_CACHELINE) + 1;
-   size = (sizeof (mongoc_counters_t) +
-           (LAST_COUNTER * sizeof (mongoc_counter_info_t)) +
+   size = (sizeof (mongoc_counters_t) + (LAST_COUNTER * sizeof (mongoc_counter_info_t)) +
            (n_cpu * n_groups * sizeof (mongoc_counter_slots_t)));
 
 #ifdef BSON_OS_UNIX
@@ -138,7 +137,9 @@ _mongoc_counters_cleanup (void)
       int pid;
 
       pid = getpid ();
-      bson_snprintf (name, sizeof name, "/mongoc-%u", pid);
+      // Truncation is OK.
+      int req = bson_snprintf (name, sizeof name, "/mongoc-%d", pid);
+      BSON_ASSERT (req > 0);
       shm_unlink (name);
 #endif
    }
@@ -170,14 +171,14 @@ mongoc_counters_alloc (size_t size)
    }
 
    pid = getpid ();
-   bson_snprintf (name, sizeof name, "/mongoc-%u", pid);
+   // Truncation is OK.
+   int req = bson_snprintf (name, sizeof name, "/mongoc-%d", pid);
+   BSON_ASSERT (req > 0);
 
 #ifndef O_NOFOLLOW
 #define O_NOFOLLOW 0
 #endif
-   if (-1 == (fd = shm_open (name,
-                             O_CREAT | O_EXCL | O_RDWR,
-                             S_IRUSR | S_IWUSR | O_NOFOLLOW))) {
+   if (-1 == (fd = shm_open (name, O_CREAT | O_EXCL | O_RDWR, S_IRUSR | S_IWUSR | O_NOFOLLOW))) {
       goto fail_noclean;
    }
 
@@ -234,11 +235,8 @@ skip_shm:
  * Returns: The offset to the data for the counters values.
  */
 static size_t
-mongoc_counters_register (mongoc_counters_t *counters,
-                          uint32_t num,
-                          const char *category,
-                          const char *name,
-                          const char *description)
+mongoc_counters_register (
+   mongoc_counters_t *counters, uint32_t num, const char *category, const char *name, const char *description)
 {
    mongoc_counter_info_t *infos;
    char *segment;
@@ -263,15 +261,13 @@ mongoc_counters_register (mongoc_counters_t *counters,
    infos = (mongoc_counter_info_t *) (segment + counters->infos_offset);
    infos = &infos[counters->n_counters];
    infos->slot = num % SLOTS_PER_CACHELINE;
-   infos->offset =
-      (counters->values_offset +
-       ((num / SLOTS_PER_CACHELINE) * n_cpu * sizeof (mongoc_counter_slots_t)));
+   infos->offset = (counters->values_offset + ((num / SLOTS_PER_CACHELINE) * n_cpu * sizeof (mongoc_counter_slots_t)));
 
    bson_strncpy (infos->category, category, sizeof infos->category);
    bson_strncpy (infos->name, name, sizeof infos->name);
    bson_strncpy (infos->description, description, sizeof infos->description);
 
-   bson_atomic_thread_fence ();
+   mcommon_atomic_thread_fence ();
 
    counters->n_counters++;
 
@@ -308,9 +304,8 @@ _mongoc_counters_init (void)
 
    BSON_ASSERT ((counters->values_offset % 64) == 0);
 
-#define COUNTER(ident, Category, Name, Desc)            \
-   off = mongoc_counters_register (                     \
-      counters, COUNTER_##ident, Category, Name, Desc); \
+#define COUNTER(ident, Category, Name, Desc)                                         \
+   off = mongoc_counters_register (counters, COUNTER_##ident, Category, Name, Desc); \
    __mongoc_counter_##ident.cpus = (mongoc_counter_slots_t *) (segment + off);
 #include "mongoc-counters.defs"
 #undef COUNTER
@@ -322,7 +317,7 @@ _mongoc_counters_init (void)
     * we have initialized the rest of the counters. Don't forget our memory
     * barrier to prevent compiler reordering.
     */
-   bson_atomic_thread_fence ();
+   mcommon_atomic_thread_fence ();
    counters->size = (uint32_t) size;
 #endif
 }
