@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-present MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,11 +33,10 @@
 #include "mongoc-read-prefs-private.h"
 #include "mongoc-rpc-private.h"
 
-#include <bson-dsl.h>
+#include <common-bson-dsl-private.h>
 
 static bool
-_mongoc_cursor_monitor_legacy_get_more (mongoc_cursor_t *cursor,
-                                        mongoc_server_stream_t *server_stream)
+_mongoc_cursor_monitor_legacy_get_more (mongoc_cursor_t *cursor, mongoc_server_stream_t *server_stream)
 {
    bson_t doc;
    char *db;
@@ -101,9 +100,7 @@ _mongoc_cursor_monitor_legacy_query (mongoc_cursor_t *cursor,
    /* simulate a MongoDB 3.2+ "find" command */
    _mongoc_cursor_prepare_find_command (cursor, filter, &doc);
 
-   bsonBuildAppend (
-      cursor->opts,
-      insert (doc, not(key ("serverId", "maxAwaitTimeMS", "sessionId"))));
+   bsonBuildAppend (cursor->opts, insert (doc, not(key ("serverId", "maxAwaitTimeMS", "sessionId"))));
 
    r = _mongoc_cursor_monitor_command (cursor, server_stream, &doc, "find");
 
@@ -125,9 +122,7 @@ _mongoc_cursor_op_getmore_send (mongoc_cursor_t *cursor,
    BSON_ASSERT_PARAM (server_stream);
    BSON_ASSERT_PARAM (rpc);
 
-   const int32_t n_return = (flags & MONGOC_OP_QUERY_FLAG_TAILABLE_CURSOR) != 0
-                               ? 0
-                               : _mongoc_n_return (cursor);
+   const int32_t n_return = (flags & MONGOC_OP_QUERY_FLAG_TAILABLE_CURSOR) != 0 ? 0 : _mongoc_n_return (cursor);
 
    {
       int32_t message_length = 0;
@@ -135,16 +130,12 @@ _mongoc_cursor_op_getmore_send (mongoc_cursor_t *cursor,
       message_length += mcd_rpc_header_set_message_length (rpc, 0);
       message_length += mcd_rpc_header_set_request_id (rpc, request_id);
       message_length += mcd_rpc_header_set_response_to (rpc, 0);
-      message_length +=
-         mcd_rpc_header_set_op_code (rpc, MONGOC_OP_CODE_GET_MORE);
+      message_length += mcd_rpc_header_set_op_code (rpc, MONGOC_OP_CODE_GET_MORE);
 
       message_length += sizeof (int32_t); // ZERO
-      message_length +=
-         mcd_rpc_op_get_more_set_full_collection_name (rpc, cursor->ns);
-      message_length +=
-         mcd_rpc_op_get_more_set_number_to_return (rpc, n_return);
-      message_length +=
-         mcd_rpc_op_get_more_set_cursor_id (rpc, cursor->cursor_id);
+      message_length += mcd_rpc_op_get_more_set_full_collection_name (rpc, cursor->ns);
+      message_length += mcd_rpc_op_get_more_set_number_to_return (rpc, n_return);
+      message_length += mcd_rpc_op_get_more_set_cursor_id (rpc, cursor->cursor_id);
 
       mcd_rpc_message_set_length (rpc, message_length);
    }
@@ -153,8 +144,7 @@ _mongoc_cursor_op_getmore_send (mongoc_cursor_t *cursor,
       return false;
    }
 
-   if (!mongoc_cluster_legacy_rpc_sendv_to_server (
-          &cursor->client->cluster, rpc, server_stream, &cursor->error)) {
+   if (!mongoc_cluster_legacy_rpc_sendv_to_server (&cursor->client->cluster, rpc, server_stream, &cursor->error)) {
       return false;
    }
 
@@ -162,8 +152,7 @@ _mongoc_cursor_op_getmore_send (mongoc_cursor_t *cursor,
 }
 
 void
-_mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
-                           mongoc_cursor_response_legacy_t *response)
+_mongoc_cursor_op_getmore (mongoc_cursor_t *cursor, mongoc_cursor_response_legacy_t *response)
 {
    BSON_ASSERT_PARAM (cursor);
    BSON_ASSERT_PARAM (response);
@@ -172,8 +161,7 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
 
    const int64_t started = bson_get_monotonic_time ();
 
-   mongoc_server_stream_t *const server_stream =
-      _mongoc_cursor_fetch_stream (cursor);
+   mongoc_server_stream_t *const server_stream = _mongoc_cursor_fetch_stream (cursor);
 
    if (!server_stream) {
       GOTO (done);
@@ -185,13 +173,11 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
    }
    mongoc_cluster_t *const cluster = &cursor->client->cluster;
 
-   const int32_t request_id = cursor->in_exhaust
-                                 ? mcd_rpc_header_get_request_id (response->rpc)
-                                 : ++cluster->request_id;
+   const int32_t request_id =
+      cursor->in_exhaust ? mcd_rpc_header_get_request_id (response->rpc) : ++cluster->request_id;
 
    if (!cursor->in_exhaust &&
-       !_mongoc_cursor_op_getmore_send (
-          cursor, server_stream, request_id, flags, response->rpc)) {
+       !_mongoc_cursor_op_getmore_send (cursor, server_stream, request_id, flags, response->rpc)) {
       GOTO (fail);
    }
 
@@ -199,11 +185,7 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
    _mongoc_buffer_clear (&response->buffer, false);
    cursor->cursor_id = 0;
 
-   if (!_mongoc_client_recv (cursor->client,
-                             response->rpc,
-                             &response->buffer,
-                             server_stream,
-                             &cursor->error)) {
+   if (!_mongoc_client_recv (cursor->client, response->rpc, &response->buffer, server_stream, &cursor->error)) {
       GOTO (fail);
    }
 
@@ -212,8 +194,7 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
       bson_set_error (&cursor->error,
                       MONGOC_ERROR_PROTOCOL,
                       MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid opcode for OP_GET_MORE: expected %" PRId32
-                      ", got %" PRId32,
+                      "invalid opcode for OP_GET_MORE: expected %" PRId32 ", got %" PRId32,
                       MONGOC_OP_CODE_REPLY,
                       op_code);
       GOTO (fail);
@@ -224,17 +205,14 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
       bson_set_error (&cursor->error,
                       MONGOC_ERROR_PROTOCOL,
                       MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid response_to for OP_GET_MORE: expected %" PRId32
-                      ", got %" PRId32,
+                      "invalid response_to for OP_GET_MORE: expected %" PRId32 ", got %" PRId32,
                       request_id,
                       response_to);
       GOTO (fail);
    }
 
-   if (!mcd_rpc_message_check_ok (response->rpc,
-                                  cursor->client->error_api_version,
-                                  &cursor->error,
-                                  &cursor->error_doc)) {
+   if (!mcd_rpc_message_check_ok (
+          response->rpc, cursor->client->error_api_version, &cursor->error, &cursor->error_doc)) {
       GOTO (fail);
    }
 
@@ -263,8 +241,7 @@ _mongoc_cursor_op_getmore (mongoc_cursor_t *cursor,
    GOTO (done);
 
 fail:
-   _mongoc_cursor_monitor_failed (
-      cursor, bson_get_monotonic_time () - started, server_stream, "getMore");
+   _mongoc_cursor_monitor_failed (cursor, bson_get_monotonic_time () - started, server_stream, "getMore");
 
 done:
    mongoc_server_stream_cleanup (server_stream);
@@ -298,21 +275,17 @@ done:
    } while (false)
 
 
-#define OPT_ERR(_msg)                                   \
-   do {                                                 \
-      bson_set_error (&cursor->error,                   \
-                      MONGOC_ERROR_COMMAND,             \
-                      MONGOC_ERROR_COMMAND_INVALID_ARG, \
-                      _msg);                            \
-      return NULL;                                      \
+#define OPT_ERR(_msg)                                                                                \
+   do {                                                                                              \
+      bson_set_error (&cursor->error, MONGOC_ERROR_COMMAND, MONGOC_ERROR_COMMAND_INVALID_ARG, _msg); \
+      return NULL;                                                                                   \
    } while (false)
 
 
-#define OPT_BSON_ERR(_msg)                                                    \
-   do {                                                                       \
-      bson_set_error (                                                        \
-         &cursor->error, MONGOC_ERROR_BSON, MONGOC_ERROR_BSON_INVALID, _msg); \
-      return NULL;                                                            \
+#define OPT_BSON_ERR(_msg)                                                                 \
+   do {                                                                                    \
+      bson_set_error (&cursor->error, MONGOC_ERROR_BSON, MONGOC_ERROR_BSON_INVALID, _msg); \
+      return NULL;                                                                         \
    } while (false)
 
 
@@ -453,14 +426,10 @@ _mongoc_cursor_parse_opts_for_op_query (mongoc_cursor_t *cursor,
        * maxAwaitTimeMS is handled in _mongoc_cursor_prepare_getmore_command
        * sessionId is used to retrieve the mongoc_client_session_t
        */
-      else if (strcmp (key, MONGOC_CURSOR_SINGLE_BATCH) &&
-               strcmp (key, MONGOC_CURSOR_LIMIT) &&
-               strcmp (key, MONGOC_CURSOR_BATCH_SIZE) &&
-               strcmp (key, MONGOC_CURSOR_EXHAUST) &&
-               strcmp (key, MONGOC_CURSOR_NO_CURSOR_TIMEOUT) &&
-               strcmp (key, MONGOC_CURSOR_OPLOG_REPLAY) &&
-               strcmp (key, MONGOC_CURSOR_TAILABLE) &&
-               strcmp (key, MONGOC_CURSOR_MAX_AWAIT_TIME_MS)) {
+      else if (strcmp (key, MONGOC_CURSOR_SINGLE_BATCH) && strcmp (key, MONGOC_CURSOR_LIMIT) &&
+               strcmp (key, MONGOC_CURSOR_BATCH_SIZE) && strcmp (key, MONGOC_CURSOR_EXHAUST) &&
+               strcmp (key, MONGOC_CURSOR_NO_CURSOR_TIMEOUT) && strcmp (key, MONGOC_CURSOR_OPLOG_REPLAY) &&
+               strcmp (key, MONGOC_CURSOR_TAILABLE) && strcmp (key, MONGOC_CURSOR_MAX_AWAIT_TIME_MS)) {
          /* pass unrecognized options to server, prefixed with $ */
          PUSH_DOLLAR_QUERY ();
          dollar_modifier = bson_strdup_printf ("$%s", key);
@@ -508,15 +477,14 @@ _mongoc_cursor_op_query_find_send (mongoc_cursor_t *cursor,
    bson_t fields = BSON_INITIALIZER;
    int32_t skip;
    int32_t flags;
-   const bson_t *const query_ptr = _mongoc_cursor_parse_opts_for_op_query (
-      cursor, server_stream, filter, &query, &fields, &flags, &skip);
+   const bson_t *const query_ptr =
+      _mongoc_cursor_parse_opts_for_op_query (cursor, server_stream, filter, &query, &fields, &flags, &skip);
 
    if (!query_ptr) {
       GOTO (done);
    }
 
-   assemble_query (
-      cursor->read_prefs, server_stream, query_ptr, flags, &result);
+   assemble_query (cursor->read_prefs, server_stream, query_ptr, flags, &result);
 
    {
       int32_t message_length = 0;
@@ -527,17 +495,13 @@ _mongoc_cursor_op_query_find_send (mongoc_cursor_t *cursor,
       message_length += mcd_rpc_header_set_op_code (rpc, MONGOC_OP_CODE_QUERY);
 
       message_length += mcd_rpc_op_query_set_flags (rpc, result.flags);
-      message_length +=
-         mcd_rpc_op_query_set_full_collection_name (rpc, cursor->ns);
+      message_length += mcd_rpc_op_query_set_full_collection_name (rpc, cursor->ns);
       message_length += mcd_rpc_op_query_set_number_to_skip (rpc, skip);
-      message_length +=
-         mcd_rpc_op_query_set_number_to_return (rpc, _mongoc_n_return (cursor));
-      message_length += mcd_rpc_op_query_set_query (
-         rpc, bson_get_data (result.assembled_query));
+      message_length += mcd_rpc_op_query_set_number_to_return (rpc, _mongoc_n_return (cursor));
+      message_length += mcd_rpc_op_query_set_query (rpc, bson_get_data (result.assembled_query));
 
       if (!bson_empty (&fields)) {
-         message_length += mcd_rpc_op_query_set_return_fields_selector (
-            rpc, bson_get_data (&fields));
+         message_length += mcd_rpc_op_query_set_return_fields_selector (rpc, bson_get_data (&fields));
       }
 
       mcd_rpc_message_set_length (rpc, message_length);
@@ -547,8 +511,7 @@ _mongoc_cursor_op_query_find_send (mongoc_cursor_t *cursor,
       GOTO (done);
    }
 
-   if (!mongoc_cluster_legacy_rpc_sendv_to_server (
-          &cursor->client->cluster, rpc, server_stream, &cursor->error)) {
+   if (!mongoc_cluster_legacy_rpc_sendv_to_server (&cursor->client->cluster, rpc, server_stream, &cursor->error)) {
       GOTO (done);
    }
 
@@ -563,9 +526,7 @@ done:
 }
 
 bool
-_mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
-                              bson_t *filter,
-                              mongoc_cursor_response_legacy_t *response)
+_mongoc_cursor_op_query_find (mongoc_cursor_t *cursor, bson_t *filter, mongoc_cursor_response_legacy_t *response)
 {
    BSON_ASSERT_PARAM (cursor);
    BSON_ASSERT_PARAM (filter);
@@ -575,8 +536,7 @@ _mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
 
    bool ret = false;
 
-   mongoc_server_stream_t *const server_stream =
-      _mongoc_cursor_fetch_stream (cursor);
+   mongoc_server_stream_t *const server_stream = _mongoc_cursor_fetch_stream (cursor);
 
    if (!server_stream) {
       RETURN (false);
@@ -586,19 +546,14 @@ _mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
    const int32_t request_id = ++cursor->client->cluster.request_id;
    mcd_rpc_message *const rpc = mcd_rpc_message_new ();
 
-   if (!_mongoc_cursor_op_query_find_send (
-          cursor, server_stream, request_id, filter, rpc)) {
+   if (!_mongoc_cursor_op_query_find_send (cursor, server_stream, request_id, filter, rpc)) {
       GOTO (done);
    }
 
    mcd_rpc_message_reset (rpc);
    _mongoc_buffer_clear (&response->buffer, false);
 
-   if (!_mongoc_client_recv (cursor->client,
-                             response->rpc,
-                             &response->buffer,
-                             server_stream,
-                             &cursor->error)) {
+   if (!_mongoc_client_recv (cursor->client, response->rpc, &response->buffer, server_stream, &cursor->error)) {
       GOTO (done);
    }
 
@@ -607,8 +562,7 @@ _mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
       bson_set_error (&cursor->error,
                       MONGOC_ERROR_PROTOCOL,
                       MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid opcode for OP_QUERY: expected %" PRId32
-                      ", got %" PRId32,
+                      "invalid opcode for OP_QUERY: expected %" PRId32 ", got %" PRId32,
                       MONGOC_OP_CODE_REPLY,
                       op_code);
       GOTO (done);
@@ -619,17 +573,14 @@ _mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
       bson_set_error (&cursor->error,
                       MONGOC_ERROR_PROTOCOL,
                       MONGOC_ERROR_PROTOCOL_INVALID_REPLY,
-                      "invalid response_to for OP_QUERY: expected %" PRId32
-                      ", got %" PRId32,
+                      "invalid response_to for OP_QUERY: expected %" PRId32 ", got %" PRId32,
                       request_id,
                       response_to);
       GOTO (done);
    }
 
-   if (!mcd_rpc_message_check_ok (response->rpc,
-                                  cursor->client->error_api_version,
-                                  &cursor->error,
-                                  &cursor->error_doc)) {
+   if (!mcd_rpc_message_check_ok (
+          response->rpc, cursor->client->error_api_version, &cursor->error, &cursor->error_doc)) {
       GOTO (done);
    }
 
@@ -664,8 +615,7 @@ _mongoc_cursor_op_query_find (mongoc_cursor_t *cursor,
 
 done:
    if (!ret) {
-      _mongoc_cursor_monitor_failed (
-         cursor, bson_get_monotonic_time () - started, server_stream, "find");
+      _mongoc_cursor_monitor_failed (cursor, bson_get_monotonic_time () - started, server_stream, "find");
    }
 
    mcd_rpc_message_destroy (rpc);
@@ -684,8 +634,7 @@ _mongoc_cursor_response_legacy_init (mongoc_cursor_response_legacy_t *response)
 
 
 void
-_mongoc_cursor_response_legacy_destroy (
-   mongoc_cursor_response_legacy_t *response)
+_mongoc_cursor_response_legacy_destroy (mongoc_cursor_response_legacy_t *response)
 {
    if (response->reader) {
       bson_reader_destroy (response->reader);
