@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -46,8 +46,7 @@
 #endif
 
 #ifndef MONGOC_NO_AUTOMATIC_GLOBALS
-#pragma message( \
-   "Configure the driver with ENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF.\
+#pragma message("Configure the driver with ENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF.\
  Automatic cleanup is deprecated and will be removed in version 2.0.")
 #endif
 
@@ -109,14 +108,14 @@ static BSON_ONCE_FUN (_mongoc_do_init)
 #ifdef MONGOC_ENABLE_SASL_CYRUS
    /* The following functions should not use tracing, as they may be invoked
     * before mongoc_log_set_handler() can complete. */
-   sasl_set_mutex (mongoc_cyrus_mutex_alloc,
-                   mongoc_cyrus_mutex_lock,
-                   mongoc_cyrus_mutex_unlock,
-                   mongoc_cyrus_mutex_free);
+   sasl_set_mutex (
+      mongoc_cyrus_mutex_alloc, mongoc_cyrus_mutex_lock, mongoc_cyrus_mutex_unlock, mongoc_cyrus_mutex_free);
 
+   MC_DISABLE_CAST_FUNCTION_TYPE_STRICT_WARNING_BEGIN
    sasl_callback_t callbacks[] = {// Include callback to disable loading plugins.
                                   {SASL_CB_VERIFYFILE, SASL_CALLBACK_FN (_mongoc_cyrus_verifyfile_cb), NULL},
                                   {SASL_CB_LIST_END}};
+   MC_DISABLE_CAST_FUNCTION_TYPE_STRICT_WARNING_END
 
    status = sasl_client_init (callbacks);
    BSON_ASSERT (status == SASL_OK);
