@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,11 +30,9 @@ typedef struct _mongoc_matcher_t mongoc_matcher_t;
 
 
 MONGOC_EXPORT (mongoc_matcher_t *)
-mongoc_matcher_new (const bson_t *query,
-                    bson_error_t *error) BSON_GNUC_DEPRECATED;
+mongoc_matcher_new (const bson_t *query, bson_error_t *error) BSON_GNUC_WARN_UNUSED_RESULT BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (bool)
-mongoc_matcher_match (const mongoc_matcher_t *matcher,
-                      const bson_t *document) BSON_GNUC_DEPRECATED;
+mongoc_matcher_match (const mongoc_matcher_t *matcher, const bson_t *document) BSON_GNUC_DEPRECATED;
 MONGOC_EXPORT (void)
 mongoc_matcher_destroy (mongoc_matcher_t *matcher) BSON_GNUC_DEPRECATED;
 
