@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,16 +14,14 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_SERVER_STREAM_H
 #define MONGOC_SERVER_STREAM_H
 
 #include "mongoc-config.h"
 
-#if !defined (MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
-#include <bson.h>
+#include <bson/bson.h>
 
 #include "mongoc-topology-description-private.h"
 #include "mongoc-server-description-private.h"
@@ -31,16 +29,23 @@
 
 BSON_BEGIN_DECLS
 
-typedef struct _mongoc_server_stream_t
-{
-   mongoc_topology_description_type_t  topology_type;
-   mongoc_server_description_t        *sd;            /* owned */
-   mongoc_stream_t                    *stream;        /* borrowed */
+typedef struct _mongoc_server_stream_t {
+   mongoc_topology_description_type_t topology_type;
+   mongoc_server_description_t *sd; // owned
+   bson_t cluster_time;             // owned
+   mongoc_stream_t *stream;         // borrowed
+   // If the stream was created in a way that may have overwritten the user's
+   // readPreference, we need to know if server selection forced that change.
+   bool must_use_primary;
+   // True if this server stream was acquired during a retry attempt triggered
+   // by a network error establishing an initial connection. Used to avoid
+   // further retry attempts.
+   bool retry_attempted;
 } mongoc_server_stream_t;
 
 
 mongoc_server_stream_t *
-mongoc_server_stream_new (mongoc_topology_description_type_t topology_type,
+mongoc_server_stream_new (const mongoc_topology_description_t *td,
                           mongoc_server_description_t *sd,
                           mongoc_stream_t *stream);
 
