@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,13 +14,11 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_SOCKET_PRIVATE_H
 #define MONGOC_SOCKET_PRIVATE_H
 
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
 #include "mongoc-socket.h"
 
 BSON_BEGIN_DECLS
@@ -33,12 +31,11 @@ struct _mongoc_socket_t {
 #endif
    int errno_;
    int domain;
+   int pid;
 };
 
 mongoc_socket_t *
-mongoc_socket_accept_ex (mongoc_socket_t *sock,
-                         int64_t expire_at,
-                         uint16_t *port);
+mongoc_socket_accept_ex (mongoc_socket_t *sock, int64_t expire_at, uint16_t *port);
 
 BSON_END_DECLS
 
