@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,13 +28,15 @@
 #include <Mstcpip.h>
 #include <process.h>
 #endif
+#include <common-cmp-private.h>
+
+#include <inttypes.h>
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "socket"
 
 
-#define OPERATION_EXPIRED(expire_at) \
-   ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
+#define OPERATION_EXPIRED(expire_at) ((expire_at >= 0) && (expire_at < (bson_get_monotonic_time ())))
 
 
 /* either struct sockaddr or void, depending on platform */
@@ -203,8 +205,7 @@ _mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
       } else {
          timeout_tv.tv_sec = timeout / 1000;
          timeout_tv.tv_usec = (timeout % 1000) * 1000;
-         ret = select (
-            0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
+         ret = select (0 /*unused*/, &read_fds, &write_fds, &error_fds, &timeout_tv);
       }
       if (ret == SOCKET_ERROR) {
          _mongoc_socket_capture_errno (sock);
@@ -220,8 +221,7 @@ _mongoc_socket_wait (mongoc_socket_t *sock, /* IN */
       if (ret > 0) {
 /* Something happened, so return that */
 #ifdef _WIN32
-         return (FD_ISSET (sock->sd, &read_fds) ||
-                 FD_ISSET (sock->sd, &write_fds));
+         return (FD_ISSET (sock->sd, &read_fds) || FD_ISSET (sock->sd, &write_fds));
 #else
          RETURN (0 != (pfd.revents & events));
 #endif
@@ -292,7 +292,6 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
    struct pollfd *pfds;
 #endif
    int ret;
-   int i;
 
    ENTRY;
 
@@ -303,7 +302,7 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
    FD_ZERO (&write_fds);
    FD_ZERO (&error_fds);
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       if (sds[i].events & POLLIN) {
          FD_SET (sds[i].socket->sd, &read_fds);
       }
@@ -325,7 +324,7 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
       return -1;
    }
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       if (FD_ISSET (sds[i].socket->sd, &read_fds)) {
          sds[i].revents = POLLIN;
       } else if (FD_ISSET (sds[i].socket->sd, &write_fds)) {
@@ -339,14 +338,14 @@ mongoc_socket_poll (mongoc_socket_poll_t *sds, /* IN */
 #else
    pfds = (struct pollfd *) bson_malloc (sizeof (*pfds) * nsds);
 
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       pfds[i].fd = sds[i].socket->sd;
       pfds[i].events = sds[i].events | POLLERR | POLLHUP;
       pfds[i].revents = 0;
    }
 
    ret = poll (pfds, nsds, timeout);
-   for (i = 0; i < nsds; i++) {
+   for (size_t i = 0u; i < nsds; i++) {
       sds[i].revents = pfds[i].revents;
    }
 
@@ -372,8 +371,7 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
    DWORD type;
    DWORD data;
    DWORD data_size = sizeof data;
-   const char *reg_key =
-      "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
+   const char *reg_key = "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters";
    keepalive.onoff = true;
    keepalive.keepalivetime = MONGODB_KEEPIDLE * 1000;
    keepalive.keepaliveinterval = MONGODB_KEEPALIVEINTVL * 1000;
@@ -389,16 +387,13 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
     * change the default value by setting the registry values.
     */
 
-   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) ==
-       ERROR_SUCCESS) {
+   if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, reg_key, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS) {
       /* https://technet.microsoft.com/en-us/library/cc957549.aspx */
       DWORD default_keepalivetime = 7200000; /* 2 hours */
       /* https://technet.microsoft.com/en-us/library/cc957548.aspx */
       DWORD default_keepaliveinterval = 1000; /* 1 second */
 
-      if (RegQueryValueEx (
-             hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) ==
-          ERROR_SUCCESS) {
+      if (RegQueryValueEx (hKey, "KeepAliveTime", NULL, &type, (LPBYTE) &data, &data_size) == ERROR_SUCCESS) {
          if (type == REG_DWORD && data < keepalive.keepalivetime) {
             keepalive.keepalivetime = data;
          }
@@ -406,12 +401,7 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
          keepalive.keepalivetime = default_keepalivetime;
       }
 
-      if (RegQueryValueEx (hKey,
-                           "KeepAliveInterval",
-                           NULL,
-                           &type,
-                           (LPBYTE) &data,
-                           &data_size) == ERROR_SUCCESS) {
+      if (RegQueryValueEx (hKey, "KeepAliveInterval", NULL, &type, (LPBYTE) &data, &data_size) == ERROR_SUCCESS) {
          if (type == REG_DWORD && data < keepalive.keepaliveinterval) {
             keepalive.keepaliveinterval = data;
          }
@@ -420,15 +410,8 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
       }
       RegCloseKey (hKey);
    }
-   if (WSAIoctl (sd,
-                 SIO_KEEPALIVE_VALS,
-                 &keepalive,
-                 sizeof keepalive,
-                 NULL,
-                 0,
-                 &lpcbBytesReturned,
-                 NULL,
-                 NULL) == SOCKET_ERROR) {
+   if (WSAIoctl (sd, SIO_KEEPALIVE_VALS, &keepalive, sizeof keepalive, NULL, 0, &lpcbBytesReturned, NULL, NULL) ==
+       SOCKET_ERROR) {
       TRACE ("%s", "Could not set keepalive values");
    } else {
       TRACE ("%s", "KeepAlive values updated");
@@ -472,25 +455,15 @@ _mongoc_socket_set_sockopt_if_less (int sd, int name, int value)
 
    optlen = sizeof optval;
    if (getsockopt (sd, IPPROTO_TCP, name, (char *) &optval, &optlen)) {
-      TRACE ("Getting '%s' failed, errno: %d",
-             _mongoc_socket_sockopt_value_to_name (name),
-             errno);
+      TRACE ("Getting '%s' failed, errno: %d", _mongoc_socket_sockopt_value_to_name (name), errno);
    } else {
-      TRACE ("'%s' is %d, target value is %d",
-             _mongoc_socket_sockopt_value_to_name (name),
-             optval,
-             value);
+      TRACE ("'%s' is %d, target value is %d", _mongoc_socket_sockopt_value_to_name (name), optval, value);
       if (optval > value) {
          optval = value;
-         if (setsockopt (
-                sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
-            TRACE ("Setting '%s' failed, errno: %d",
-                   _mongoc_socket_sockopt_value_to_name (name),
-                   errno);
+         if (setsockopt (sd, IPPROTO_TCP, name, (char *) &optval, sizeof optval)) {
+            TRACE ("Setting '%s' failed, errno: %d", _mongoc_socket_sockopt_value_to_name (name), errno);
          } else {
-            TRACE ("'%s' value changed to %d",
-                   _mongoc_socket_sockopt_value_to_name (name),
-                   optval);
+            TRACE ("'%s' value changed to %d", _mongoc_socket_sockopt_value_to_name (name), optval);
          }
       }
    }
@@ -508,8 +481,7 @@ _mongoc_socket_setkeepalive_nix (int sd)
 #endif
 
 #ifdef TCP_KEEPINTVL
-   _mongoc_socket_set_sockopt_if_less (
-      sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
+   _mongoc_socket_set_sockopt_if_less (sd, TCP_KEEPINTVL, MONGODB_KEEPALIVEINTVL);
 #else
    TRACE ("%s", "TCP_KEEPINTVL not available");
 #endif
@@ -534,8 +506,7 @@ _mongoc_socket_setkeepalive (int sd) /* IN */
 
    ENTRY;
 #ifdef SO_KEEPALIVE
-   if (!setsockopt (
-          sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
+   if (!setsockopt (sd, SOL_SOCKET, SO_KEEPALIVE, (char *) &optval, sizeof optval)) {
       TRACE ("%s", "Setting SO_KEEPALIVE");
 #ifdef _WIN32
       _mongoc_socket_setkeepalive_windows (sd);
@@ -557,7 +528,7 @@ static bool
 #ifdef _WIN32
 _mongoc_socket_setnodelay (SOCKET sd) /* IN */
 #else
-_mongoc_socket_setnodelay (int sd)   /* IN */
+_mongoc_socket_setnodelay (int sd) /* IN */
 #endif
 {
 #ifdef _WIN32
@@ -570,8 +541,7 @@ _mongoc_socket_setnodelay (int sd)   /* IN */
    ENTRY;
 
    errno = 0;
-   ret = setsockopt (
-      sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
+   ret = setsockopt (sd, IPPROTO_TCP, TCP_NODELAY, (char *) &optval, sizeof optval);
 
 #ifdef _WIN32
    if (ret == SOCKET_ERROR) {
@@ -879,8 +849,7 @@ mongoc_socket_connect (mongoc_socket_t *sock,       /* IN */
    if (failed && try_again) {
       if (_mongoc_socket_wait (sock, POLLOUT, expire_at)) {
          optval = -1;
-         ret = getsockopt (
-            sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
+         ret = getsockopt (sock->sd, SOL_SOCKET, SO_ERROR, (char *) &optval, &optlen);
          if ((ret == 0) && (optval == 0)) {
             RETURN (0);
          } else {
@@ -1098,8 +1067,7 @@ again:
 #endif
    if (failed) {
       _mongoc_socket_capture_errno (sock);
-      if (_mongoc_socket_errno_is_again (sock) &&
-          _mongoc_socket_wait (sock, POLLIN, expire_at)) {
+      if (_mongoc_socket_errno_is_again (sock) && _mongoc_socket_wait (sock, POLLIN, expire_at)) {
          GOTO (again);
       }
    }
@@ -1215,8 +1183,6 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
                                size_t iovcnt)         /* IN */
 {
    ssize_t ret = 0;
-   size_t i;
-   ssize_t wrote;
 
    ENTRY;
 
@@ -1224,11 +1190,13 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
    BSON_ASSERT (iov);
    BSON_ASSERT (iovcnt);
 
-   for (i = 0; i < iovcnt; i++) {
-      wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
+   for (size_t i = 0u; i < iovcnt; i++) {
 #ifdef _WIN32
+      BSON_ASSERT (mcommon_in_range_unsigned (int, iov[i].iov_len));
+      const int wrote = send (sock->sd, iov[i].iov_base, (int) iov[i].iov_len, 0);
       if (wrote == SOCKET_ERROR) {
 #else
+      const ssize_t wrote = send (sock->sd, iov[i].iov_base, iov[i].iov_len, 0);
       if (wrote == -1) {
 #endif
          _mongoc_socket_capture_errno (sock);
@@ -1241,7 +1209,7 @@ _mongoc_socket_try_sendv_slow (mongoc_socket_t *sock, /* IN */
 
       ret += wrote;
 
-      if (wrote != iov[i].iov_len) {
+      if (mcommon_cmp_not_equal_su (wrote, iov[i].iov_len)) {
          RETURN (ret);
       }
    }
@@ -1291,16 +1259,13 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
    DUMP_IOVEC (sendbuf, iov, iovcnt);
 
 #ifdef _WIN32
-   ret = WSASend (
-      sock->sd, (LPWSABUF) iov, iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
-   TRACE ("WSASend sent: %ld (out of: %zu), ret: %d",
-          dwNumberofBytesSent,
-          iov->iov_len,
-          ret);
+   BSON_ASSERT (mcommon_in_range_unsigned (unsigned_long, iovcnt));
+   ret = WSASend (sock->sd, (LPWSABUF) iov, (DWORD) iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
+   TRACE ("WSASend sent: %lu (out of: %zu), ret: %d", dwNumberofBytesSent, iov->iov_len, ret);
 #else
    memset (&msg, 0, sizeof msg);
    msg.msg_iov = iov;
-   msg.msg_iovlen = (int) iovcnt;
+   msg.msg_iovlen = iovcnt;
    ret = sendmsg (sock->sd,
                   &msg,
 #ifdef MSG_NOSIGNAL
@@ -1308,7 +1273,7 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
 #else
                   0);
 #endif
-   TRACE ("Send %ld out of %ld bytes", ret, iov->iov_len);
+   TRACE ("Send %zd out of %zu bytes", ret, iov->iov_len);
 #endif
 
 
@@ -1388,8 +1353,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
 
    for (;;) {
       sent = _mongoc_socket_try_sendv (sock, &iov[cur], iovcnt - cur);
-      TRACE (
-         "Sent %zd (of %zu) out of iovcnt=%zu", sent, iov[cur].iov_len, iovcnt);
+      TRACE ("Sent %zd (of %zu) out of iovcnt=%zu", sent, iov[cur].iov_len, iovcnt);
 
       /*
        * If we failed with anything other than EAGAIN or EWOULDBLOCK,
@@ -1414,9 +1378,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * Subtract the sent amount from what we still need to send.
           */
          while ((cur < iovcnt) && (sent >= (ssize_t) iov[cur].iov_len)) {
-            TRACE ("still got bytes left: sent -= iov_len: %zd -= %zu",
-                   sent,
-                   iov[cur].iov_len);
+            TRACE ("still got bytes left: sent -= iov_len: %zd -= %zu", sent, iov[cur].iov_len);
             sent -= iov[cur++].iov_len;
          }
 
@@ -1434,8 +1396,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * the number of bytes to write.
           */
          TRACE ("Seeked io_base+%zd", sent);
-         TRACE (
-            "Subtracting iov_len -= sent; %zu -= %zd", iov[cur].iov_len, sent);
+         TRACE ("Subtracting iov_len -= sent; %zu -= %zd", iov[cur].iov_len, sent);
          iov[cur].iov_base = ((char *) iov[cur].iov_base) + sent;
          iov[cur].iov_len -= sent;
          TRACE ("iov_len remaining %zu", iov[cur].iov_len);
@@ -1500,8 +1461,7 @@ mongoc_socket_getnameinfo (mongoc_socket_t *sock) /* IN */
       RETURN (NULL);
    }
 
-   if (getnameinfo (
-          (struct sockaddr *) &addr, len, host, sizeof host, NULL, 0, 0)) {
+   if (getnameinfo ((struct sockaddr *) &addr, len, host, sizeof host, NULL, 0, 0)) {
       RETURN (NULL);
    }
 
@@ -1559,20 +1519,27 @@ mongoc_socket_inet_ntop (struct addrinfo *rp, /* IN */
 {
    void *ptr;
    char tmp[256];
+   int req;
 
    switch (rp->ai_family) {
    case AF_INET:
       ptr = &((struct sockaddr_in *) rp->ai_addr)->sin_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
-      bson_snprintf (buf, buflen, "ipv4 %s", tmp);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "ipv4 %s", tmp);
+      BSON_ASSERT (req > 0);
       break;
    case AF_INET6:
       ptr = &((struct sockaddr_in6 *) rp->ai_addr)->sin6_addr;
       inet_ntop (rp->ai_family, ptr, tmp, sizeof (tmp));
-      bson_snprintf (buf, buflen, "ipv6 %s", tmp);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "ipv6 %s", tmp);
+      BSON_ASSERT (req > 0);
       break;
    default:
-      bson_snprintf (buf, buflen, "unknown ip %d", rp->ai_family);
+      // Truncation is OK.
+      req = bson_snprintf (buf, buflen, "unknown ip %d", rp->ai_family);
+      BSON_ASSERT (req > 0);
       break;
    }
 }
