@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,14 +14,12 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_SSL_H
 #define MONGOC_SSL_H
 
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
-#include <bson.h>
+#include <bson/bson.h>
 
 #include "mongoc-macros.h"
 
@@ -39,12 +37,13 @@ struct _mongoc_ssl_opt_t {
    const char *crl_file;
    bool weak_cert_validation;
    bool allow_invalid_hostname;
-   void *padding[7];
+   void *internal;
+   void *padding[6];
 };
 
 
 MONGOC_EXPORT (const mongoc_ssl_opt_t *)
-mongoc_ssl_opt_get_default (void) BSON_GNUC_CONST;
+mongoc_ssl_opt_get_default (void) BSON_GNUC_PURE;
 
 
 BSON_END_DECLS
