@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,13 +14,11 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_STREAM_SOCKET_H
 #define MONGOC_STREAM_SOCKET_H
 
-#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
-#error "Only <mongoc.h> can be included directly."
-#endif
-
 #include "mongoc-macros.h"
 #include "mongoc-socket.h"
 #include "mongoc-stream.h"
@@ -33,7 +31,7 @@ typedef struct _mongoc_stream_socket_t mongoc_stream_socket_t;
 
 
 MONGOC_EXPORT (mongoc_stream_t *)
-mongoc_stream_socket_new (mongoc_socket_t *socket);
+mongoc_stream_socket_new (mongoc_socket_t *socket) BSON_GNUC_WARN_UNUSED_RESULT;
 MONGOC_EXPORT (mongoc_socket_t *)
 mongoc_stream_socket_get_socket (mongoc_stream_socket_t *stream);
 
