@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@
 #include "mongoc-stream-private.h"
 #include "mongoc-trace-private.h"
 #include "mongoc-util-private.h"
+#include <common-cmp-private.h>
 
 
 #undef MONGOC_LOG_DOMAIN
@@ -142,10 +143,7 @@ mongoc_stream_flush (mongoc_stream_t *stream)
  * Returns: the number of bytes written, or -1 upon failure.
  */
 ssize_t
-mongoc_stream_writev (mongoc_stream_t *stream,
-                      mongoc_iovec_t *iov,
-                      size_t iovcnt,
-                      int32_t timeout_msec)
+mongoc_stream_writev (mongoc_stream_t *stream, mongoc_iovec_t *iov, size_t iovcnt, int32_t timeout_msec)
 {
    ssize_t ret;
 
@@ -157,6 +155,7 @@ mongoc_stream_writev (mongoc_stream_t *stream,
 
    BSON_ASSERT (stream->writev);
 
+   // CDRIVER-4781: for backward compatibility.
    if (timeout_msec < 0) {
       timeout_msec = MONGOC_DEFAULT_TIMEOUT_MSEC;
    }
@@ -179,10 +178,7 @@ mongoc_stream_writev (mongoc_stream_t *stream,
  * Returns: -1 on failure, otherwise the number of bytes written.
  */
 ssize_t
-mongoc_stream_write (mongoc_stream_t *stream,
-                     void *buf,
-                     size_t count,
-                     int32_t timeout_msec)
+mongoc_stream_write (mongoc_stream_t *stream, void *buf, size_t count, int32_t timeout_msec)
 {
    mongoc_iovec_t iov;
    ssize_t ret;
@@ -218,11 +214,8 @@ mongoc_stream_write (mongoc_stream_t *stream,
  * Returns: the number of bytes read or -1 on failure.
  */
 ssize_t
-mongoc_stream_readv (mongoc_stream_t *stream,
-                     mongoc_iovec_t *iov,
-                     size_t iovcnt,
-                     size_t min_bytes,
-                     int32_t timeout_msec)
+mongoc_stream_readv (
+   mongoc_stream_t *stream, mongoc_iovec_t *iov, size_t iovcnt, size_t min_bytes, int32_t timeout_msec)
 {
    ssize_t ret;
 
@@ -259,11 +252,7 @@ mongoc_stream_readv (mongoc_stream_t *stream,
  * Returns: -1 on failure, otherwise the number of bytes read.
  */
 ssize_t
-mongoc_stream_read (mongoc_stream_t *stream,
-                    void *buf,
-                    size_t count,
-                    size_t min_bytes,
-                    int32_t timeout_msec)
+mongoc_stream_read (mongoc_stream_t *stream, void *buf, size_t count, size_t min_bytes, int32_t timeout_msec)
 {
    mongoc_iovec_t iov;
    ssize_t ret;
@@ -285,11 +274,7 @@ mongoc_stream_read (mongoc_stream_t *stream,
 
 
 int
-mongoc_stream_setsockopt (mongoc_stream_t *stream,
-                          int level,
-                          int optname,
-                          void *optval,
-                          mongoc_socklen_t optlen)
+mongoc_stream_setsockopt (mongoc_stream_t *stream, int level, int optname, void *optval, mongoc_socklen_t optlen)
 {
    BSON_ASSERT_PARAM (stream);
 
@@ -331,33 +316,28 @@ mongoc_stream_get_tls_stream (mongoc_stream_t *stream) /* IN */
 {
    BSON_ASSERT_PARAM (stream);
 
-   for (; stream && stream->type != MONGOC_STREAM_TLS;
-        stream = stream->get_base_stream (stream))
+   for (; stream && stream->type != MONGOC_STREAM_TLS; stream = stream->get_base_stream (stream))
       ;
 
    return stream;
 }
 
 ssize_t
-mongoc_stream_poll (mongoc_stream_poll_t *streams,
-                    size_t nstreams,
-                    int32_t timeout)
+mongoc_stream_poll (mongoc_stream_poll_t *streams, size_t nstreams, int32_t timeout)
 {
-   mongoc_stream_poll_t *poller =
-      (mongoc_stream_poll_t *) bson_malloc (sizeof (*poller) * nstreams);
+   mongoc_stream_poll_t *poller = (mongoc_stream_poll_t *) bson_malloc (sizeof (*poller) * nstreams);
 
-   int i;
    int last_type = 0;
    ssize_t rval = -1;
 
    errno = 0;
 
-   for (i = 0; i < nstreams; i++) {
+   for (size_t i = 0u; i < nstreams; i++) {
       poller[i].stream = mongoc_stream_get_root_stream (streams[i].stream);
       poller[i].events = streams[i].events;
       poller[i].revents = 0;
 
-      if (i == 0) {
+      if (i == 0u) {
          last_type = poller[i].stream->type;
       } else if (last_type != poller[i].stream->type) {
          errno = EINVAL;
@@ -373,7 +353,7 @@ mongoc_stream_poll (mongoc_stream_poll_t *streams,
    rval = poller[0].stream->poll (poller, nstreams, timeout);
 
    if (rval > 0) {
-      for (i = 0; i < nstreams; i++) {
+      for (size_t i = 0u; i < nstreams; i++) {
          streams[i].revents = poller[i].revents;
       }
    }
@@ -423,22 +403,28 @@ mongoc_stream_should_retry (mongoc_stream_t *stream)
 }
 
 bool
-_mongoc_stream_writev_full (mongoc_stream_t *stream,
-                            mongoc_iovec_t *iov,
-                            size_t iovcnt,
-                            int32_t timeout_msec,
-                            bson_error_t *error)
+_mongoc_stream_writev_full (
+   mongoc_stream_t *stream, mongoc_iovec_t *iov, size_t iovcnt, int64_t timeout_msec, bson_error_t *error)
 {
    size_t total_bytes = 0;
-   int i;
    ssize_t r;
    ENTRY;
 
-   for (i = 0; i < iovcnt; i++) {
+   for (size_t i = 0u; i < iovcnt; i++) {
       total_bytes += iov[i].iov_len;
    }
 
-   r = mongoc_stream_writev (stream, iov, iovcnt, timeout_msec);
+   if (BSON_UNLIKELY (!mcommon_in_range_signed (int32_t, timeout_msec))) {
+      // CDRIVER-4589
+      bson_set_error (error,
+                      MONGOC_ERROR_STREAM,
+                      MONGOC_ERROR_STREAM_SOCKET,
+                      "timeout_msec value %" PRId64 " exceeds supported 32-bit range",
+                      timeout_msec);
+      RETURN (false);
+   }
+
+   r = mongoc_stream_writev (stream, iov, iovcnt, (int32_t) timeout_msec);
    TRACE ("writev returned: %zd", r);
 
    if (r < 0) {
@@ -459,14 +445,14 @@ _mongoc_stream_writev_full (mongoc_stream_t *stream,
       RETURN (false);
    }
 
-   if (r != total_bytes) {
+   if (mcommon_cmp_not_equal_su (r, total_bytes)) {
       bson_set_error (error,
                       MONGOC_ERROR_STREAM,
                       MONGOC_ERROR_STREAM_SOCKET,
-                      "Failure to send all requested bytes (only sent: %" PRIu64
-                      "/%" PRId64 " in %dms) during socket delivery",
+                      "Failure to send all requested bytes (only sent: %" PRIu64 "/%zu in %" PRId64
+                      "ms) during socket delivery",
                       (uint64_t) r,
-                      (int64_t) total_bytes,
+                      total_bytes,
                       timeout_msec);
 
       RETURN (false);
