@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,18 +14,18 @@
  * limitations under the License.
  */
 
-#include "mongoc/mongoc-prelude.h"
+#include "mongoc-prelude.h"
 
 #ifndef MONGOC_WRITE_COMMAND_PRIVATE_H
 #define MONGOC_WRITE_COMMAND_PRIVATE_H
 
 #include <bson/bson.h>
 
-#include "mongoc/mongoc-client.h"
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-write-concern.h"
-#include "mongoc/mongoc-server-stream-private.h"
-#include "mongoc/mongoc-buffer-private.h"
+#include "mongoc-client.h"
+#include "mongoc-error.h"
+#include "mongoc-write-concern.h"
+#include "mongoc-server-stream-private.h"
+#include "mongoc-buffer-private.h"
 
 
 BSON_BEGIN_DECLS
@@ -37,12 +37,21 @@ struct _mongoc_crud_opts_t;
 #define MONGOC_WRITE_COMMAND_INSERT 1
 #define MONGOC_WRITE_COMMAND_UPDATE 2
 
+/* MongoDB has a extra allowance to allow updating 16mb document, as the update
+ * operators would otherwise overflow the 16mb object limit. See SERVER-10643
+ * for context. */
+#define BSON_OBJECT_ALLOWANCE (16 * 1024)
+
+#define RETRYABLE_WRITE_ERROR "RetryableWriteError"
+
 struct _mongoc_bulk_write_flags_t {
    bool ordered;
    bool bypass_document_validation;
    bool has_collation;
    bool has_multi_write;
    bool has_array_filters;
+   bool has_update_hint;
+   bool has_delete_hint;
 };
 
 
@@ -53,11 +62,6 @@ typedef struct {
    mongoc_bulk_write_flags_t flags;
    int64_t operation_id;
    bson_t cmd_opts;
-   union {
-      struct {
-         bool allow_bulk_op_insert;
-      } insert;
-   } u;
 } mongoc_write_command_t;
 
 
@@ -80,6 +84,13 @@ typedef struct {
    bool must_stop; /* The stream may have been disconnected */
    bson_error_t error;
    uint32_t upsert_append_count;
+   /* If the command initially failed with a retryable write, and selected a new
+    * primary, this contains the server id of the newly selected primary. Only
+    * applies to OP_MSG. Is left at 0 if no retry occurs. */
+   uint32_t retry_server_id;
+   /* store the raw server response if an error occured */
+   uint32_t n_errorReplies;
+   bson_t rawErrorReplies;
 } mongoc_write_result_t;
 
 
@@ -91,30 +102,27 @@ typedef enum {
 } mongoc_write_err_type_t;
 
 
-const char *
-_mongoc_command_type_to_field_name (int command_type);
-const char *
-_mongoc_command_type_to_name (int command_type);
-
 void
 _mongoc_write_command_destroy (mongoc_write_command_t *command);
 void
-_mongoc_write_command_init (bson_t *doc,
-                            mongoc_write_command_t *command,
-                            const char *collection);
+_mongoc_write_command_init (bson_t *doc, mongoc_write_command_t *command, const char *collection);
 void
 _mongoc_write_command_init_insert (mongoc_write_command_t *command,
                                    const bson_t *document,
                                    const bson_t *cmd_opts,
                                    mongoc_bulk_write_flags_t flags,
-                                   int64_t operation_id,
-                                   bool allow_bulk_op_insert);
+                                   int64_t operation_id);
+void
+_mongoc_write_command_init_insert_one_idl (mongoc_write_command_t *command,
+                                           const bson_t *document,
+                                           const bson_t *cmd_opts,
+                                           bson_t *insert_id,
+                                           int64_t operation_id);
 void
 _mongoc_write_command_init_insert_idl (mongoc_write_command_t *command,
                                        const bson_t *document,
                                        const bson_t *cmd_opts,
-                                       int64_t operation_id,
-                                       bool allow_bulk_op_insert);
+                                       int64_t operation_id);
 void
 _mongoc_write_command_init_delete (mongoc_write_command_t *command,
                                    const bson_t *selectors,
@@ -132,6 +140,7 @@ void
 _mongoc_write_command_init_update (mongoc_write_command_t *command,
                                    const bson_t *selector,
                                    const bson_t *update,
+                                   const bson_t *cmd_opts,
                                    const bson_t *opts,
                                    mongoc_bulk_write_flags_t flags,
                                    int64_t operation_id);
@@ -139,11 +148,11 @@ void
 _mongoc_write_command_init_update_idl (mongoc_write_command_t *command,
                                        const bson_t *selector,
                                        const bson_t *update,
+                                       const bson_t *cmd_opts,
                                        const bson_t *opts,
                                        int64_t operation_id);
 void
-_mongoc_write_command_insert_append (mongoc_write_command_t *command,
-                                     const bson_t *document);
+_mongoc_write_command_insert_append (mongoc_write_command_t *command, const bson_t *document);
 void
 _mongoc_write_command_update_append (mongoc_write_command_t *command,
                                      const bson_t *selector,
@@ -151,16 +160,9 @@ _mongoc_write_command_update_append (mongoc_write_command_t *command,
                                      const bson_t *opts);
 
 void
-_mongoc_write_command_delete_append (mongoc_write_command_t *command,
-                                     const bson_t *selector,
-                                     const bson_t *opts);
+_mongoc_write_command_delete_append (mongoc_write_command_t *command, const bson_t *selector, const bson_t *opts);
 
 void
-_mongoc_write_command_too_large_error (bson_error_t *error,
-                                       int32_t idx,
-                                       int32_t len,
-                                       int32_t max_bson_size);
-void
 _mongoc_write_command_execute (mongoc_write_command_t *command,
                                mongoc_client_t *client,
                                mongoc_server_stream_t *server_stream,
@@ -181,22 +183,7 @@ _mongoc_write_command_execute_idl (mongoc_write_command_t *command,
                                    mongoc_write_result_t *result);
 void
 _mongoc_write_result_init (mongoc_write_result_t *result);
-void
-_mongoc_write_result_append_upsert (mongoc_write_result_t *result,
-                                    int32_t idx,
-                                    const bson_value_t *value);
-int32_t
-_mongoc_write_result_merge_arrays (uint32_t offset,
-                                   mongoc_write_result_t *result,
-                                   bson_t *dest,
-                                   bson_iter_t *iter);
-void
-_mongoc_write_result_merge (mongoc_write_result_t *result,
-                            mongoc_write_command_t *command,
-                            const bson_t *reply,
-                            uint32_t offset);
-#define MONGOC_WRITE_RESULT_COMPLETE(_result, ...) \
-   _mongoc_write_result_complete (_result, __VA_ARGS__, NULL)
+#define MONGOC_WRITE_RESULT_COMPLETE(_result, ...) _mongoc_write_result_complete (_result, __VA_ARGS__, NULL)
 bool
 _mongoc_write_result_complete (mongoc_write_result_t *result,
                                int32_t error_api_version,
@@ -208,13 +195,11 @@ _mongoc_write_result_complete (mongoc_write_result_t *result,
 void
 _mongoc_write_result_destroy (mongoc_write_result_t *result);
 
-void
-_append_array_from_command (mongoc_write_command_t *command, bson_t *bson);
-
 mongoc_write_err_type_t
-_mongoc_write_error_get_type (bool cmd_ret,
-                              const bson_error_t *cmd_err,
-                              const bson_t *reply);
+_mongoc_write_error_get_type (bson_t *reply);
+
+bool
+_mongoc_write_error_update_if_unsupported_storage_engine (bool cmd_ret, bson_error_t *cmd_err, bson_t *reply);
 
 BSON_END_DECLS
 
